#include "sparrowshell.h"

#include "message.h"
#include "input.h"
#include "scenetree.h"
#include "mesh.h"
#include "phongmaterial.h"

const unsigned int SparrowShell::BUFFER_MAX_LENGTH = 50;
const unsigned int SparrowShell::BUFFER_DISPLAYED_NUMBER = 10;
const unsigned int SparrowShell::SCROLLBAR_PIXEL_WIDTH = 2;

SparrowShell::SparrowShell(sf::Window* window, Input* input): m_window(window),m_input(input),m_position(glm::ivec2(0,0)),m_scrollbar(this)
{
    sf::Vector2u size = m_window->getSize();
    m_dimension = glm::ivec2(size.x,size.y/2);
    Mesh* mesh = new Mesh();
    mesh->addRectangle2D(m_position,m_dimension);
    PhongMaterial *mat = new PhongMaterial();
    mat->diffuse = glm::vec3(0,0.5,0.5);
    mesh->setMaterial(mat);
    mesh->initGL();
    this->addChild(new MeshNode(mesh));
}

void SparrowShell::out(std::string s)
{
    if (m_buffer.size() == BUFFER_MAX_LENGTH)
        m_buffer.pop_back();
    m_buffer.push_front(s);
}

void SparrowShell::scrollUp(){
    if (m_index + BUFFER_DISPLAYED_NUMBER < m_buffer.size()) m_index++;
}

void SparrowShell::scrollDown(){
    if (m_index > 0) m_index--;
}

void SparrowShell::update()
{
    //TODO : update TextMesh
    m_scrollbar.update();
}

//GeometryNode* SparrowShell::getGeometryNode()
//{
//    return geometry;
//}


SparrowShell::ScrollBar::ScrollBar(SparrowShell* shell):m_shell(shell){
    m_position = glm::ivec2(m_shell->m_dimension.x - SCROLLBAR_PIXEL_WIDTH,0);
    m_dimension = glm::ivec2(SCROLLBAR_PIXEL_WIDTH,m_shell->m_dimension.y);
}

void SparrowShell::ScrollBar::update(){
    m_position.y = m_shell->m_position.y;
    m_dimension.y = m_shell->m_dimension.y;

    if (m_shell->m_buffer.size() > BUFFER_DISPLAYED_NUMBER){
        float cran = ((float)m_shell->m_dimension.y/(float)m_shell->m_buffer.size());
        int indexCursor = m_shell->m_buffer.size()-(m_shell->m_index+SCROLLBAR_PIXEL_WIDTH);
        m_position.y += (int)(cran * indexCursor);
        m_dimension.y = (int)(cran * BUFFER_DISPLAYED_NUMBER);
    }
}
